<?php /* EL PSY CONGROO */    	   			 
require_once "WxPay.Config.Interface.php";     		 	   
require_once "WxPay.Data.php";            
    			    	
class WxPayException extends Exception {    		 		 	 
	public function errorMessage()      				 	
	{    				 			
		return $this->getMessage();      		    
	}        	  	
}    			 				
     				  	
    		      
/**    	     		
 * 接口访问类，包含所有微信支付API列表的封装，类中方法为static方法，    					 		
 */    	  					
class WxPayApi    	  		 		
{    	  	 	 	
	/*      	 		 	
	 * 统一下单，WxPayUnifiedOrder中out_trade_no、body、total_fee、trade_type必填      		 	  
	 */    	   	  	
	public static function unifiedOrder($config, $inputObj, $timeOut = 6)    						  
	{     			   	
      	  			
		$url = "https://api.mch.weixin.qq.com/pay/unifiedorder";    			    	
		//检测必填参数     			 			
		if(!$inputObj->IsOut_trade_noSet()) {    					   
			throw new WxPayException("缺少统一支付接口必填参数out_trade_no！");       	  		
		}else if(!$inputObj->IsBodySet()){    	   	 	 
			throw new WxPayException("缺少统一支付接口必填参数body！");    			  	  
		}else if(!$inputObj->IsTotal_feeSet()) {    	  	 	  
			throw new WxPayException("缺少统一支付接口必填参数total_fee！");      	 	 	 
		}else if(!$inputObj->IsTrade_typeSet()) {    							 
			throw new WxPayException("缺少统一支付接口必填参数trade_type！");      	 		  
		}    	 	 	   
		      	 		 	
		//关联参数    	 					 
		if($inputObj->GetTrade_type() == "JSAPI" && !$inputObj->IsOpenidSet()){    	      	
			throw new WxPayException("统一支付接口中，缺少必填参数openid！trade_type为JSAPI时，openid为必填参数！");     	    		
		}     			  	 
		if($inputObj->GetTrade_type() == "NATIVE" && !$inputObj->IsProduct_idSet()){    	   			 
			throw new WxPayException("统一支付接口中，缺少必填参数product_id！trade_type为JSAPI时，product_id为必填参数！");    	   				
		}    	 					 
		       	  		
		//异步通知url未设置，则使用配置文件中的url     	 		 	 
		if(!$inputObj->IsNotify_urlSet() && $config->GetNotifyUrl() != ""){     	  	 	 
			$inputObj->SetNotify_url($config->GetNotifyUrl());//异步通知url       					
		}    	 				 	
		    			 		  
		$inputObj->SetAppid($config->GetAppId());//公众账号ID    		 					
		$inputObj->SetMch_id($config->GetMerchantId());//商户号    			  			
		$inputObj->SetSpbill_create_ip($_SERVER['REMOTE_ADDR']);//终端ip	   	         			 		 
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串    			 			 
              		   	
    			 		 	
		//签名    	   			 
		$inputObj->SetSign($config);    	 		   	
		$xml = $inputObj->ToXml();    			  	  
		    	 		 			
		$startTimeStamp = self::getMillisecond();//请求开始时间    	  	 			
		$response = self::postXmlCurl($config, $xml, $url, false, $timeOut);      				  
		$result = WxPayResults::Init($config, $response);    		  	 	 
		self::reportCostTime($config, $url, $startTimeStamp, $result);//上报请求花费时间      					 
		      			   
		return $result;    					 		
	}    	  	 			
	         			
	/**     		  	 	
	 * 查询订单，WxPayOrderQuery中out_trade_no、transaction_id至少填一个     			  		
	 */     		  			
	public static function orderQuery($config, $inputObj, $timeOut = 6)    	    			
	{     	  		 	
		$url = "https://api.mch.weixin.qq.com/pay/orderquery";    							 
		//检测必填参数    							 
		if(!$inputObj->IsOut_trade_noSet() && !$inputObj->IsTransaction_idSet()) {    	 			 		
			throw new WxPayException("订单查询接口中，out_trade_no、transaction_id至少填一个！");      	 		  
		}    								
		$inputObj->SetAppid($config->GetAppId());//公众账号ID     	 	  	 
		$inputObj->SetMch_id($config->GetMerchantId());//商户号    		   	 	
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串    			  	 	
		    					  	
		$inputObj->SetSign($config);//签名     	 				 
		$xml = $inputObj->ToXml();    	   	 	 
		    		 	 	  
		$startTimeStamp = self::getMillisecond();//请求开始时间     		 		  
		$response = self::postXmlCurl($config, $xml, $url, false, $timeOut);      	  		 
		$result = WxPayResults::Init($config, $response);    						  
		self::reportCostTime($config, $url, $startTimeStamp, $result);//上报请求花费时间    		    	 
		    	 	   		
		return $result;       	 			
	}    	 					 
	    	   			 
	/**      	 				
	 * 关闭订单，WxPayCloseOrder中out_trade_no必填       	 	 	
	 */        			 
	public static function closeOrder($config, $inputObj, $timeOut = 6)    	  	    
	{    	   		  
		$url = "https://api.mch.weixin.qq.com/pay/closeorder";           	
		//检测必填参数    	 		 	 	
		if(!$inputObj->IsOut_trade_noSet()) {    		 					
			throw new WxPayException("订单查询接口中，out_trade_no必填！");    						 	
		}    	 		 	  
		$inputObj->SetAppid($config->GetAppId());//公众账号ID    		 	 	  
		$inputObj->SetMch_id($config->GetMerchantId());//商户号    					 	 
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串     	  		 	
		     	 	 		 
		$inputObj->SetSign($config);//签名     	 	 	  
		$xml = $inputObj->ToXml();    	 			   
		     	 	 	  
		$startTimeStamp = self::getMillisecond();//请求开始时间    	  		 		
		$response = self::postXmlCurl($config, $xml, $url, false, $timeOut);    	 	  	  
		$result = WxPayResults::Init($config, $response);      	    	
		self::reportCostTime($config, $url, $startTimeStamp, $result);//上报请求花费时间    	 			   
		         		 
		return $result;    	  			  
	}     	   	  
     					 	
	/**     	 		 		
	 * 申请退款，WxPayRefund中out_trade_no、transaction_id至少填一个且    	    	 	
	 */     	 	   	
	public static function refund($config, $inputObj, $timeOut = 6)     	 		 		
	{      		   	
		$url = "https://api.mch.weixin.qq.com/secapi/pay/refund";     	 	 	 	
		//检测必填参数     					 	
		if(!$inputObj->IsOut_trade_noSet() && !$inputObj->IsTransaction_idSet()) {     	 		  	
			throw new WxPayException("退款申请接口中，out_trade_no、transaction_id至少填一个！");    		 		  	
		}else if(!$inputObj->IsOut_refund_noSet()){    	 		  	 
			throw new WxPayException("退款申请接口中，缺少必填参数out_refund_no！");     	      
		}else if(!$inputObj->IsTotal_feeSet()){        	 		
			throw new WxPayException("退款申请接口中，缺少必填参数total_fee！");    		 	 			
		}else if(!$inputObj->IsRefund_feeSet()){    	    	 	
			throw new WxPayException("退款申请接口中，缺少必填参数refund_fee！");    	  		  	
		}else if(!$inputObj->IsOp_user_idSet()){     	 			 	
			throw new WxPayException("退款申请接口中，缺少必填参数op_user_id！");    		 			 	
		}    		 	  		
		$inputObj->SetAppid($config->GetAppId());//公众账号ID     	 				 
		$inputObj->SetMch_id($config->GetMerchantId());//商户号       	 	  
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串     	  				
		            
		$inputObj->SetSign($config);//签名    	 				 	
		$xml = $inputObj->ToXml();      		    
		$startTimeStamp = self::getMillisecond();//请求开始时间     	  	   
		$response = self::postXmlCurl($config, $xml, $url, true, $timeOut);    					   
		$result = WxPayResults::Init($config, $response);     						 
		self::reportCostTime($config, $url, $startTimeStamp, $result);//上报请求花费时间      	  	 	
		    	  				 
		return $result;    	 	 		 	
	}    		 			 	
	    		 	    
	/**     	 	    
	 * 查询退款    	 			   
	 */    	   	  	
	public static function refundQuery($config, $inputObj, $timeOut = 6)     		 		  
	{     	 		 		
		$url = "https://api.mch.weixin.qq.com/pay/refundquery";     		 			 
		//检测必填参数      	  	 	
		if(!$inputObj->IsOut_refund_noSet() &&      	 	   
			!$inputObj->IsOut_trade_noSet() &&    	  		 		
			!$inputObj->IsTransaction_idSet() &&      	 	 		
			!$inputObj->IsRefund_idSet()) {    		 	 			
			throw new WxPayException("退款查询接口中，out_refund_no、out_trade_no、transaction_id、refund_id四个参数必填一个！");    		  	  	
		}        	 	 
		$inputObj->SetAppid($config->GetAppId());//公众账号ID     			 	  
		$inputObj->SetMch_id($config->GetMerchantId());//商户号    			  		 
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串    	 					 
		     						 
		$inputObj->SetSign($config);//签名    	  	 			
		$xml = $inputObj->ToXml();         			
		    	 	 	 	 
		$startTimeStamp = self::getMillisecond();//请求开始时间      			   
		$response = self::postXmlCurl($config, $xml, $url, false, $timeOut);    	 	 			 
		$result = WxPayResults::Init($config, $response);     	  				
		self::reportCostTime($config, $url, $startTimeStamp, $result);//上报请求花费时间     		 		  
		    			   	 
		return $result;     		 				
	}    		   	 	
	    							 
	/**    	  	 	 	
	 * 下载对账单，WxPayDownloadBill中bill_date为必填参数           	
	 */         		 
	public static function downloadBill($config, $inputObj, $timeOut = 6)    		 	 	 	
	{      						
		$url = "https://api.mch.weixin.qq.com/pay/downloadbill";      				  
		//检测必填参数    	    	 	
		if(!$inputObj->IsBill_dateSet()) {    	    		 
			throw new WxPayException("对账单接口中，缺少必填参数bill_date！");    	 		   	
		}    			 	 		
		$inputObj->SetAppid($config->GetAppId());//公众账号ID    	 		  	 
		$inputObj->SetMch_id($config->GetMerchantId());//商户号      	  			
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串        	   
		      	 	   
		$inputObj->SetSign($config);//签名     	  	 		
		$xml = $inputObj->ToXml();    	 	  			
		        			 
		$response = self::postXmlCurl($config, $xml, $url, false, $timeOut);    	 			 		
		if(substr($response, 0 , 5) == "<xml>"){     	 	 	 	
			return "";       				 
		}    	  		  	
		return $response;    	 		 		 
	}    	  		  	
	      	  		 
	/**    	  	 	 	
	 * 提交被扫支付API    	  	 	  
	 */     				 		
	public static function micropay($config, $inputObj, $timeOut = 10)     		  	 	
	{    		  	   
		$url = "https://api.mch.weixin.qq.com/pay/micropay";       	 		 
		//检测必填参数    	 	 	 		
		if(!$inputObj->IsBodySet()) {    		 		 	 
			throw new WxPayException("提交被扫支付API接口中，缺少必填参数body！");      	 		 	
		} else if(!$inputObj->IsOut_trade_noSet()) {    	 		 			
			throw new WxPayException("提交被扫支付API接口中，缺少必填参数out_trade_no！");    			 	 		
		} else if(!$inputObj->IsTotal_feeSet()) {    	 						
			throw new WxPayException("提交被扫支付API接口中，缺少必填参数total_fee！");     					  
		} else if(!$inputObj->IsAuth_codeSet()) {    	   	  	
			throw new WxPayException("提交被扫支付API接口中，缺少必填参数auth_code！");    			  		 
		}     		 		  
		     		 	  	
		$inputObj->SetSpbill_create_ip($_SERVER['REMOTE_ADDR']);//终端ip     	 		   
		$inputObj->SetAppid($config->GetAppId());//公众账号ID      	 				
		$inputObj->SetMch_id($config->GetMerchantId());//商户号     		 				
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串    			  		 
		     		   		
		$inputObj->SetSign($config);//签名       	 	 	
		$xml = $inputObj->ToXml();    	  	 	 	
		     	 	  	 
		$startTimeStamp = self::getMillisecond();//请求开始时间    		 			  
		$response = self::postXmlCurl($config, $xml, $url, false, $timeOut);     	  	  	
		$result = WxPayResults::Init($config, $response);    				    
		self::reportCostTime($config, $url, $startTimeStamp, $result);//上报请求花费时间    								
		      	 		 	
		return $result;         		 
	}    		 	    
	    	 		  	 
	/**     				   
	 * 撤销订单API接口，WxPayReverse中参数out_trade_no和transaction_id必须填写一个     					  
	 */    	 	 	 		
	public static function reverse($config, $inputObj, $timeOut = 6)    			  		 
	{          		
		$url = "https://api.mch.weixin.qq.com/secapi/pay/reverse";       	 	 	
		//检测必填参数      	  	 	
		if(!$inputObj->IsOut_trade_noSet() && !$inputObj->IsTransaction_idSet()) {    							 
			throw new WxPayException("撤销订单API接口中，参数out_trade_no和transaction_id必须填写一个！");    							 
		}    			  	  
		     	   	  
		$inputObj->SetAppid($config->GetAppId());//公众账号ID    		  	 		
		$inputObj->SetMch_id($config->GetMerchantId());//商户号     							
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串    	 				 	
		     	 	    
		$inputObj->SetSign($config);//签名       					
		$xml = $inputObj->ToXml();     		   	 
		       	  		
		$startTimeStamp = self::getMillisecond();//请求开始时间    	 	 	 	 
		$response = self::postXmlCurl($config, $xml, $url, true, $timeOut);          		
		$result = WxPayResults::Init($config, $response);    		  	 	 
		self::reportCostTime($config, $url, $startTimeStamp, $result);//上报请求花费时间     	 		  	
		    	    	 	
		return $result;    				   	
	}    	  	   	
	    			  	 	
	/**     	  	 		
	 * 测速上报，该方法内部封装在report中，使用时请注意异常流程    	  			 	
	 */     	  	 	 
	public static function report($config, $inputObj, $timeOut = 1)    	 		 	  
	{    		    		
		$url = "https://api.mch.weixin.qq.com/payitil/report";    		 	 	  
		//检测必填参数    		 	  		
		if(!$inputObj->IsInterface_urlSet()) {      	    	
			throw new WxPayException("接口URL，缺少必填参数interface_url！");      	 		  
		} if(!$inputObj->IsReturn_codeSet()) {    				 			
			throw new WxPayException("返回状态码，缺少必填参数return_code！");     	  				
		} if(!$inputObj->IsResult_codeSet()) {     	 	   	
			throw new WxPayException("业务结果，缺少必填参数result_code！");      	 		  
		} if(!$inputObj->IsUser_ipSet()) {      	  			
			throw new WxPayException("访问接口IP，缺少必填参数user_ip！");    	  	 		 
		} if(!$inputObj->IsExecute_time_Set()) {      	     
			throw new WxPayException("接口耗时，缺少必填参数execute_time_！");    	 	  			
		}     				   
		$inputObj->SetAppid($config->GetAppId());//公众账号ID     		   	 
		$inputObj->SetMch_id($config->GetMerchantId());//商户号    		 	   	
		$inputObj->SetUser_ip($_SERVER['REMOTE_ADDR']);//终端ip    	 	 	  	
		$inputObj->SetTime(date("YmdHis"));//商户上报时间	     					   
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串    					 		
		     					 	
		$inputObj->SetSign($config);//签名    	 		   	
		$xml = $inputObj->ToXml();    		  	 		
		    	 	 		 	
		$startTimeStamp = self::getMillisecond();//请求开始时间      			   
		$response = self::postXmlCurl($config, $xml, $url, false, $timeOut);     	  	   
		return $response;    		 	  	 
	}    					 	 
	    		 		  	
	/**    		   	 	
	 *        	   	
	 * 生成二维码规则,模式一生成支付二维码        			 
	 */    	 		 	 	
	public static function bizpayurl($config, $inputObj, $timeOut = 6)    	    		 
	{    		 	    
		if(!$inputObj->IsProduct_idSet()){    	 			  	
			throw new WxPayException("生成二维码，缺少必填参数product_id！");    		      
		}    		 				 
      				 	
		$inputObj->SetAppid($config->GetAppId());//公众账号ID    	   	  	
		$inputObj->SetMch_id($config->GetMerchantId());//商户号    	  		  	
		$inputObj->SetTime_stamp(time());//时间戳	     		      
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串    		  				
		      		    
		$inputObj->SetSign($config);//签名    	  		 	 
		       	  	 
		return $inputObj->GetValues();     		 		 	
	}    		     	
	     	 					
	/**    			 				
	 *       	  	 	
	 * 转换短链接    		    		
	 * 该接口主要用于扫码原生支付模式一中的二维码链接转成短链接(weixin://wxpay/s/XXXXXX)，    		 		  	
	 */    		 			  
	public static function shorturl($config, $inputObj, $timeOut = 6)      	 	   
	{    		  				
		$url = "https://api.mch.weixin.qq.com/tools/shorturl";    		    		
		//检测必填参数      	   	 
		if(!$inputObj->IsLong_urlSet()) {    	   				
			throw new WxPayException("需要转换的URL，签名用原串，传输需URL encode！");    		  	   
		}        	  	
		$inputObj->SetAppid($config->GetAppId());//公众账号ID    		  			 
		$inputObj->SetMch_id($config->GetMerchantId());//商户号       	 	 	
		$inputObj->SetNonce_str(self::getNonceStr());//随机字符串     	 	  		
		    	 			   
		$inputObj->SetSign($config);//签名    					 	 
		$xml = $inputObj->ToXml();      	 				
		     		     
		$startTimeStamp = self::getMillisecond();//请求开始时间    		 					
		$response = self::postXmlCurl($config, $xml, $url, false, $timeOut);        		 	
		$result = WxPayResults::Init($config, $response);     	 			 	
		self::reportCostTime($config, $url, $startTimeStamp, $result);//上报请求花费时间    		   			
		    			   		
		return $result;    	  		 	 
	}    	   	 	 
	        	   
 	/**        		 	
 	 * 支付结果通用通知    			 		  
 	 */      			 	 
	public static function notify($config, $callback, &$msg)    				 	  
	{    	 		 	 	
		if (!isset($GLOBALS['HTTP_RAW_POST_DATA'])) {      	  	  
			# 如果没有数据，直接返回失败    			 				
			return false;    	  		 	 
		}        	   
        		 	
		//如果返回成功则验证签名    	 	     
		try {    	  	 		 
			//获取通知的数据    	 		 	  
			$xml = $GLOBALS['HTTP_RAW_POST_DATA'];     		 		 	
			$result = WxPayNotifyResults::Init($config, $xml);     		   	 
		} catch (WxPayException $e){    		  	 	 
			$msg = $e->errorMessage();    	   	  	
			return false;    	 	  	 	
		}    	 	  	  
		       	   	
		return call_user_func($callback, $result);    	   				
	}     		 	 		
	    					 	 
	/**      		 			
	 * 产生随机字符串，不长于32位    	 				 	
	 */          	 
	public static function getNonceStr($length = 32)     				    
	{    	 	 	 	 
		$chars = "abcdefghijklmnopqrstuvwxyz0123456789";      	   	   
		$str ="";     						 
		for ( $i = 0; $i < $length; $i++ )  {        	  	 	
			$str .= substr($chars, mt_rand(0, strlen($chars)-1), 1);       			 		 
		}       				 	
		return $str;    	  	   	
	}        	 		
	     					 	
	/**       		 		
	 * 直接输出xml    		   		 
	 */     				 	 
	public static function replyNotify($xml)     	 	    
	{    		    		
		echo $xml;    	   				
	}     		 	  	
	     		 			 
	/**    	   	  	
	 * 上报数据， 上报的时候将屏蔽所有异常流程     				  	
	 */    	 	 	 	 
	private static function reportCostTime($config, $url, $startTimeStamp, $data)    		 	  	 
	{      						
		//如果不需要上报数据      	  	  
		$reportLevenl = $config->GetReportLevenl();    		   		 
		if($reportLevenl == 0){        		 	
			return;     	   	 	
		}       	   	 
		//如果仅失败上报       				 
		if($reportLevenl == 1 &&    	   				
			 array_key_exists("return_code", $data) &&    	 	 				
			 $data["return_code"] == "SUCCESS" &&      		    
			 array_key_exists("result_code", $data) &&    	 	  	 	
			 $data["result_code"] == "SUCCESS")     	  	  	
		 {     	   			
		 	return;     	  	 		
		 }     			   	
		     	 	 	  	
		//上报逻辑    						  
		$endTimeStamp = self::getMillisecond();     	   	 	
		$objInput = new WxPayReport();    			 		  
		$objInput->SetInterface_url($url);    		    	 
		$objInput->SetExecute_time_($endTimeStamp - $startTimeStamp);    		  			 
		//返回状态码    		 	 	 	
		if(array_key_exists("return_code", $data)){    				 	 	
			$objInput->SetReturn_code($data["return_code"]);    	 						
		}    			 		 	
		//返回信息     	 					
		if(array_key_exists("return_msg", $data)){     	  	   
			$objInput->SetReturn_msg($data["return_msg"]);    	  			  
		}    		  	 		
		//业务结果      	 	   
		if(array_key_exists("result_code", $data)){    		 			 	
			$objInput->SetResult_code($data["result_code"]);      	 	  	
		}    					 		
		//错误代码    		  				
		if(array_key_exists("err_code", $data)){    	 		  	 
			$objInput->SetErr_code($data["err_code"]);    			     
		}    	 		 	  
		//错误代码描述     	  	  	
		if(array_key_exists("err_code_des", $data)){        		  
			$objInput->SetErr_code_des($data["err_code_des"]);    	    			
		}     			 			
		//商户订单号    	 				 	
		if(array_key_exists("out_trade_no", $data)){    			  			
			$objInput->SetOut_trade_no($data["out_trade_no"]);     					  
		}    		   	  
		//设备号    	 	   	 
		if(array_key_exists("device_info", $data)){     	 	  		
			$objInput->SetDevice_info($data["device_info"]);    	 			 		
		}        		 	
		      					 
		try{     	 	 	 	
			self::report($config, $objInput);     	      
		} catch (WxPayException $e){     		 	   
			//不做任何处理     		    	
		}    					 		
	}      		    
    		      
	/**    	 	 		  
	 * 以post方式提交xml到对应的接口url    						  
	 */    			 		 	
	private static function postXmlCurl($config, $xml, $url, $useCert = false, $second = 30)       	  	 
	{		     			 	 	
		$ch = curl_init();    	  	  	 
		$curlVersion = curl_version();    			 			 
		$ua = "WXPaySDK/3.0.9 (".PHP_OS.") PHP/".PHP_VERSION." CURL/".$curlVersion['version']." "    	  				 
		.$config->GetMerchantId();       		   
       	 	  
		//设置超时    	   				
		curl_setopt($ch, CURLOPT_TIMEOUT, $second);    	 		   	
    			 		 	
		$proxyHost = "0.0.0.0";       			 	
		$proxyPort = 0;    	       
		$config->GetProxy($proxyHost, $proxyPort);     			 	  
		//如果有配置代理这里就设置代理    	 		 	 	
		if($proxyHost != "0.0.0.0" && $proxyPort != 0){          	 
			curl_setopt($ch,CURLOPT_PROXY, $proxyHost);    					 	 
			curl_setopt($ch,CURLOPT_PROXYPORT, $proxyPort);    				 			
		}     			    
		curl_setopt($ch,CURLOPT_URL, $url);     		    	
		curl_setopt($ch,CURLOPT_SSL_VERIFYPEER,false);    			 				
		curl_setopt($ch,CURLOPT_SSL_VERIFYHOST,2);//严格校验     	 	 		 
		curl_setopt($ch,CURLOPT_USERAGENT, $ua);      	  		 	
		//设置header    			   	 
		curl_setopt($ch, CURLOPT_HEADER, FALSE);      			   
		//要求结果为字符串且输出到屏幕上    		   			
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);      	 	  	
	      		  		
		if($useCert == true){    				   	
			//设置证书    	 	  	 	
			//使用证书：cert 与 key 分别属于两个.pem文件    	  	   	
			//证书文件请放入服务器的非web目录下    							 
			$sslCertPath = "";       					
			$sslKeyPath = "";     		 		 	
			$config->GetSSLCertPath($sslCertPath, $sslKeyPath);         	 	
			curl_setopt($ch,CURLOPT_SSLCERTTYPE,'PEM');          		
			curl_setopt($ch,CURLOPT_SSLCERT, $sslCertPath);     	 	 	 	
			curl_setopt($ch,CURLOPT_SSLKEYTYPE,'PEM');    		    	 
			curl_setopt($ch,CURLOPT_SSLKEY, $sslKeyPath);      		   	
		}    				 			
		//post提交方式    						  
		curl_setopt($ch, CURLOPT_POST, TRUE);     			  	 
		curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);    		 	  	 
		//运行curl     	 		   
		$data = curl_exec($ch);       	  	 
		//返回结果     	 	  		
		if($data){    	     	 
			curl_close($ch);    		      
			return $data;      			  	
		} else {     		      
			$error = curl_errno($ch);    	  	 	 	
			curl_close($ch);      	   	 
			throw new WxPayException("curl出错，错误码:$error");    		 					
		}    	   	 		
	}      	   		
	    	   		 	
	/**    	 		  	 
	 * 获取毫秒级别的时间戳    	       
	 */    	   	 		
	private static function getMillisecond()     		 				
	{    		  		 	
		//获取毫秒的时间戳    		 				 
		$time = explode ( " ", microtime () );    	 			   
		$time = $time[1] . ($time[0] * 1000);    		 	 			
		$time2 = explode( ".", $time );    	  			  
		$time = $time2[0];     			 	  
		return $time;     	 	 	  
	}     	  		 	
}      		 	  
     				 	 
      			 		