// 获取按钮排序
function xiurr_code_get_btn(data) {
    let btn = '<div class="code-button-box">' +
        '<span class="code-button-notice"></span>';
    for (let k in data) {
        switch (k) {
            case 'full':
                btn += '<div class="code-btn code-btn-full ' + (data[k] === 'on' ? '' : 'hide') + '" title="全屏"><i class="code-iconfont">&#xeb99;</i></div>';
                break;
            case 'copy':
                btn += '<div class="code-btn code-btn-copy ' + (data[k] === 'on' ? '' : 'hide') + '" title="复制"><i class="code-iconfont">&#xeb4e;</i></div>';
                break;
            case 'close':
                btn += '<div class="code-btn code-btn-close ' + (data[k] === 'on' ? '' : 'hide') + '" title="收起"><i class="code-iconfont">&#xeb8b;</i></div>';
                break;
        }
    }
    btn += '</div>';
    return btn;
}

function xiurr_code_patch_language(str) {
    let res = /language-([^ ;"']*)/.exec(str) || /brush:([^ ;"']*)/.exec(str);
    return res !== null ? res[1] : '';
}

hljs.addPlugin({
    'after:highlightElement': ({el, result}) => {
        // 两种情况会走进来: 1. 当没有预定义代码类型时 2. 当预定义了代码类型且定义的类型插件能正常识别
        if (result.language !== undefined) { // result.language.toUpperCase()
            $(el).parent().parent().attr('data-language', result.language);
        }
    }
});

window.xiurrCode = {
    highlightAll: function () {
        // 创建外壳
        this.createPreHTML();
        // 给代码着色并进行事件绑定
        this.highlightEl();
    },

    // 给pre套壳子
    createPreHTML: function () {
        // 渲染完毕后就会加上 data-xiurr-code="finish" 这个参数
        $('pre:not([data-xiurr-code="finish"])').each(function () {
            let that = $(this);
            let cDom = that.find('code');
            // 识别是否预定义了代码块的代码类型; 如language-php,brush:php
            let res = xiurr_code_patch_language(that.attr('class')); // brush:php || language-python
            if (res === '' && cDom.length > 0) {
                // 有些编辑器是在code标签上预定义了代码类型
                res = xiurr_code_patch_language(cDom.attr('class'));
            }
            res !== '' && that.addClass('language-' + res);

            // 拼接代码块结构 toUpperCase()
            that.wrap('<div data-language="' + res + '" class="' + `code-wrapper hljs ${window.XIURR_CODE_CONFIG.border_style} ${window.XIURR_CODE_CONFIG.show_shadow} ${window.XIURR_CODE_CONFIG.show_type} ${window.XIURR_CODE_CONFIG.show_line} ${window.XIURR_CODE_CONFIG.show_line_number} ${window.XIURR_CODE_CONFIG.code_word_wrap}` + '"></div>');
            if (cDom.length <= 0) {
                that.html('<code>' + that.html() + '</code>')
            }
        });
    },

    // 给代码着色并进行事件绑定
    highlightEl: function () {
        let _this = this;
        let high_limit = parseInt(window.XIURR_CODE_CONFIG.code_height_limit);

        $('pre:not([data-xiurr-code="finish"]) code').each(function (index, element) {
            let that = $(this)
                , wrapper = that.parents(".code-wrapper")
                , pre = wrapper.children('pre')
                , btn = xiurr_code_get_btn(window.XIURR_CODE_CONFIG.code_group_btn)
                , msg = '<div class="code-msg" id="' + index + '"></div>';

            // 代码块着色
            hljs.highlightElement(element);

            // 替换标签 作用好像是为了能显示行号
            $(this).html("<ul class='xiurr-code-ul'><li>" + $(this).html().trim().replace(/\n/g, "\n</li><li>") + "\n</li></ul>" + btn + msg);
            // 计算高度
            wrapper.attr('mh', pre.outerHeight() + 'px');

            // 添加查看更多及高度限制;固定高度，高度不够，不出现滚动条
            if (high_limit > 0 && pre.outerHeight() > high_limit) {
                if (window.XIURR_CODE_CONFIG.code_limit_btn === 'on') {
                    // 多加30的高度，避免真实高度只是比设定高度多一点点而出现按钮
                    if (pre.outerHeight() > high_limit + 30) {
                        wrapper.prepend('<div class="code-btn-expand"><i class="code-iconfont">&#xeb04;</i></div>');
                        pre.css('height', high_limit + 'px');
                    }
                } else {
                    pre.css('height', high_limit + 'px');
                }
            }

            // 配置默认是否处于关闭状态
            if (window.XIURR_CODE_CONFIG.code_close === 'on') {
                wrapper.attr('close', 'true').children('pre').css('max-height', '0').scrollTop(0);
            }

            // 按钮绑定事件
            _this.bindButton(wrapper);

            // 给pre加上完成渲染的属性
            pre.attr("data-xiurr-code", "finish");
        });
    },

    // 按钮事件绑定
    bindButton: function (codeWrapper) {
        // 折叠按钮
        codeWrapper.on('click', '.code-btn-close', function () {
            let wrapper = $(this).parents('.code-wrapper'),
                pre = wrapper.children('pre'),
                btn = wrapper.find('.code-btn-expand');
            if (wrapper.attr('close') === 'true') {
                // 展开
                btn.show();
                wrapper.attr('close', 'false').children('pre').css('max-height', '');
            } else {
                // 折叠
                wrapper.attr('close', 'true').children('pre').css('max-height', '0').scrollTop(0);
                wrapper.find('.code-btn-expand').hide();
            }
        })

        // 全屏按钮
        codeWrapper.on('click', '.code-btn-full', function () {
            let that = $(this)
                , wrapper = that.parents(".code-wrapper")
                , pre = wrapper.children('pre')

            if (that.hasClass('full')) {
                // 取消全屏
                pre.css('max-height', this.maxHeight).css('height', this.height);
                that.removeClass('full').children().text("\ueb99");
                wrapper.removeClass('code-block-fullscreen').find('.code-btn-close').show();
            } else {
                // 全屏
                this.height = pre.prop('style').height
                this.maxHeight = pre.prop('style').maxHeight;
                pre.css({'max-height': '', 'height': ''});
                that.addClass('full').children().text("\ueb98");
                wrapper.addClass('code-block-fullscreen').find('.code-btn-close').hide();
            }
        });

        // 展开按钮
        codeWrapper.on('click', '.code-btn-expand', function () {
            $(this).addClass('expand-done').next('pre').css('height', '');
        });
    },
}

$(function () {

    // 进行全局渲染
    window.xiurrCode.highlightAll();

    // 绑定复制按钮（动态生成的也可以绑定）
    var clip = new ClipboardJS(".code-btn-copy", {
        target: function (trigger) {
            return trigger.parentNode.parentNode
        }
    });

    let msg_content = '<div class="code-msg-content"><div class="code-msg-content-success"><i class="code-msg-icon"><svg width="16" height="16" viewBox="0 0 48 48" fill="none" xmlns="http://www.w3.org/2000/svg"><rect width="48" height="48" fill="white" fill-opacity="0.01"></rect><path d="M24 4L29.2533 7.83204L35.7557 7.81966L37.7533 14.0077L43.0211 17.8197L41 24L43.0211 30.1803L37.7533 33.9923L35.7557 40.1803L29.2533 40.168L24 44L18.7467 40.168L12.2443 40.1803L10.2467 33.9923L4.97887 30.1803L7 24L4.97887 17.8197L10.2467 14.0077L12.2443 7.81966L18.7467 7.83204L24 4Z" fill="#52c41a" stroke="#52c41a" stroke-width="4" stroke-linecap="round" stroke-linejoin="round"></path><path d="M17 24L22 29L32 19" stroke="#FFF" stroke-width="4" stroke-linecap="round" stroke-linejoin="round"></path></svg></i><span>复制成功！</span></div></div>'
        , remove_id = {}
    clip.on("success", function (e) {
        let mDom = $(e.trigger).parents('code').find('.code-msg')
            , index = mDom.attr('id');
        if (remove_id[index] === undefined) {
            remove_id[index] = {'clickCopyOut': null, 'messageMoveOut': null};
        }

        if (mDom.html() === '') {
            mDom.html(msg_content).css('animation-name', 'MessageMoveIn');
            $(e.trigger).children().text("\ueb75").css('color', '#35cd4b');
        }
        // 清除之前点击的倒计时
        clearTimeout(remove_id[index].clickCopyOut);
        clearTimeout(remove_id[index].messageMoveOut);
        // 动画清除
        remove_id[index].clickCopyOut = setTimeout(function () {
            $(e.trigger).children().text("\ueb4e").css('color', '');
            mDom.css('animation-name', 'MessageMoveOut');
            remove_id[index].messageMoveOut = setTimeout(function () {
                mDom.css('animation-name', '').html('');
            }, 280);
        }, 1500);
        // 清除文本选中状态
        e.clearSelection();
    });
    clip.on("error", function (e) {
        alert("复制失败")
    })
});