<?php

// 系统初始化
require_once __DIR__ . '/../../../../zb_system/function/c_system_base.php';

// 加载系统
$zbp->Load();

// 声明返回JSON
header('Content-Type:application/json; charset=utf-8');

// 检测管理员权限
if (!$zbp->CheckRights('root')) {
    exit(json_show(0, "没有访问权限!"));
}

// 校验 csrfToken
// CheckHTTPRefererValid();
if (GetVars('csrfToken')) {
    if (!$zbp->VerifyCSRFToken(GetVars('csrfToken'), 'xiurr_code')) {
        exit(json_show(0, "非法访问!"));
    }
} else {
    exit(json_show(0, "非法访问!"));
}

// 检测主题/插件启用状态
if (!$zbp->CheckPlugin('xiurr_code')) {
    exit(json_show(0, "插件未启用!"));
}

// 判断请求类型
switch (strtoupper($_SERVER['REQUEST_METHOD'])) {
    case "GET":
        $action = GetVars('action', 'GET');
        break;
    case "POST":
        // 配置类型
        $type = GetVars('action');
        if (!in_array($type, ["save"])) {
            exit(json_show(0, "保存类型有误"));
        }

        $zbp->Config('xiurr_code')->config = json_encode([
            'border_style'      => GetVars('border_style', 'POST', ''), // 外观样式
            'highlight_style'   => GetVars('highlight_style', 'POST', ''), // 高亮样式
            'show_shadow'       => GetVars('show_shadow', 'POST', ''), // 外观阴影
            'show_type'         => GetVars('show_type', 'POST', ''), // 代码类型
            'show_line'         => GetVars('show_line', 'POST', ''), // 行数分割线
            'show_line_number'  => GetVars('show_line_number', 'POST', ''), // 行数字
            'code_word_wrap'    => GetVars('code_word_wrap', 'POST', ''), // 代码换行
            'code_group_btn'    => GetVars('code_group_btn', 'POST', ['full' => 'on', 'copy' => 'on', 'close' => 'on']), // 显示按钮
            'code_close'        => GetVars('code_close', 'POST', 'off'), // 是否默认处于折叠状态
            'code_height_limit' => GetVars('code_height_limit', 'POST', 0), // 高度限制
            'code_limit_btn'    => GetVars('code_limit_btn', 'POST', 'off'), // 是否显示展开按钮
            'keep_config'       => GetVars('keep_config', 'POST', 'on'), // 卸载时是否保留配置
            'code_style'        => GetVars('code_style', 'POST', ''), // 拓展样式
        ], JSON_UNESCAPED_UNICODE);

        // 保存配置
        $zbp->SaveConfig('xiurr_code');
        exit(json_show(1, "配置保存成功"));
        break;
}

// 返回JSON
function json_show($code = 0, $message = "success", $data = [])
{
    return json_encode(["code" => $code, "message" => $message, "data" => $data], JSON_UNESCAPED_UNICODE);
}